@@ -14,25 +14,13 @@
 
 import abc
 import threading
+import uuid
 
-from oslo_config import cfg
 from oslo_utils import excutils
 from oslo_utils import timeutils
-import six
-
 
 from oslo_messaging import exceptions
 
-base_opts = [
-    cfg.IntOpt('rpc_conn_pool_size', default=30,
-               deprecated_group='DEFAULT',
-               help='Size of RPC connection pool.'),
-    cfg.IntOpt('conn_pool_min_size', default=2,
-               help='The pool size limit for connections expiration policy'),
-    cfg.IntOpt('conn_pool_ttl', default=1200,
-               help='The time-to-live in sec of idle connections in the pool')
-]
-
 
 def batch_poll_helper(func):
     """Decorator to poll messages in batch
@@ -41,6 +29,7 @@ def batch_poll_helper(func):
     :py:meth:`PollStyleListener.poll` implementation that only polls for a
     single message per call.
     """
+
     def wrapper(in_self, timeout=None, batch_size=1, batch_timeout=None):
         incomings = []
         driver_prefetch = in_self.prefetch_size
@@ -57,6 +46,7 @@ def batch_poll_helper(func):
                     break
 
         return incomings
+
     return wrapper
 
 
@@ -64,8 +54,7 @@ class TransportDriverError(exceptions.MessagingException):
     """Base class for transport driver specific exceptions."""
 
 
-@six.add_metaclass(abc.ABCMeta)
-class IncomingMessage(object):
+class IncomingMessage(metaclass=abc.ABCMeta):
     """The IncomingMessage class represents a single message received from the
     messaging backend. Instances of this class are passed to up a server's
     messaging processing logic. The backend driver must provide a concrete
@@ -78,10 +67,14 @@ class IncomingMessage(object):
     :type message: dict
     """
 
-    def __init__(self, ctxt, message):
+    def __init__(self, ctxt, message, msg_id=None):
         self.ctxt = ctxt
         self.message = message
         self.client_timeout = None
+        if msg_id is None:
+            self.msg_id = str(uuid.uuid4())
+        else:
+            self.msg_id = msg_id
 
     def acknowledge(self):
         """Called by the server to acknowledge receipt of the message. When
@@ -115,8 +108,7 @@ class IncomingMessage(object):
         """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class RpcIncomingMessage(IncomingMessage):
+class RpcIncomingMessage(IncomingMessage, metaclass=abc.ABCMeta):
     """The RpcIncomingMessage represents an RPC request message received from
     the backend. This class must be used for RPC calls that return a value to
     the caller.
@@ -169,8 +161,7 @@ class RpcIncomingMessage(IncomingMessage):
         """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PollStyleListener(object):
+class PollStyleListener(metaclass=abc.ABCMeta):
     """A PollStyleListener is used to transfer received messages to a server
     for processing. A polling pattern is used to retrieve messages.  A
     PollStyleListener uses a separate thread to run the polling loop.  A
@@ -226,8 +217,7 @@ class PollStyleListener(object):
         pass
 
 
-@six.add_metaclass(abc.ABCMeta)
-class Listener(object):
+class Listener(metaclass=abc.ABCMeta):
     """A Listener is used to transfer incoming messages from the driver to a
     server for processing.  A callback is used by the driver to transfer the
     messages.
@@ -244,9 +234,9 @@ class Listener(object):
         all backend implementations.
     :type prefetch_size: int
     """
+
     def __init__(self, batch_size, batch_timeout,
                  prefetch_size=-1):
-
         self.on_incoming_callback = None
         self.batch_timeout = batch_timeout
         self.prefetch_size = prefetch_size
@@ -283,8 +273,9 @@ class PollStyleListenerAdapter(Listener):
     """A Listener that uses a PollStyleListener for message transfer. A
     dedicated thread is created to do message polling.
     """
+
     def __init__(self, poll_style_listener, batch_size, batch_timeout):
-        super(PollStyleListenerAdapter, self).__init__(
+        super().__init__(
             batch_size, batch_timeout, poll_style_listener.prefetch_size
         )
         self._poll_style_listener = poll_style_listener
@@ -293,7 +284,7 @@ class PollStyleListenerAdapter(Listener):
         self._started = False
 
     def start(self, on_incoming_callback):
-        super(PollStyleListenerAdapter, self).start(on_incoming_callback)
+        super().start(on_incoming_callback)
         self._started = True
         self._listen_thread.start()
 
@@ -320,14 +311,13 @@ class PollStyleListenerAdapter(Listener):
         self._started = False
         self._poll_style_listener.stop()
         self._listen_thread.join()
-        super(PollStyleListenerAdapter, self).stop()
+        super().stop()
 
     def cleanup(self):
         self._poll_style_listener.cleanup()
 
 
-@six.add_metaclass(abc.ABCMeta)
-class BaseDriver(object):
+class BaseDriver(metaclass=abc.ABCMeta):
     """Defines the backend driver interface. Each backend driver implementation
     must provide a concrete derivation of this class implementing the backend
     specific logic for its public methods.
@@ -364,7 +354,7 @@ class BaseDriver(object):
     @abc.abstractmethod
     def send(self, target, ctxt, message,
              wait_for_reply=None, timeout=None, call_monitor_timeout=None,
-             retry=None):
+             retry=None, transport_options=None):
         """Send a message to the given target and optionally wait for a reply.
         This method is used by the RPC client when sending RPC requests to a
         server.
@@ -434,6 +424,10 @@ class BaseDriver(object):
         :type call_monitor_timeout: float
         :param retry: maximum message send attempts permitted
         :type retry: int
+        :param transport_options: additional parameters to configure the driver
+                                  for example to send parameters as "mandatory"
+                                  flag in RabbitMQ
+        :type transport_options: dictionary
         :returns: A reply message or None if no reply expected
         :raises: :py:exc:`MessagingException`, any exception thrown by the
             remote server when executing the RPC call.
