@@ -1,4 +1,3 @@
-
 # Copyright 2013 Red Hat, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,6 +12,29 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import logging
+import queue
+import threading
+
+from oslo_utils import eventletutils
+from oslo_utils import importutils
+
+LOG = logging.getLogger(__name__)
+
+eventlet = importutils.try_import('eventlet')
+if eventlet and eventletutils.is_monkey_patched("thread"):
+    # Here we initialize module with the native python threading module
+    # if it was already monkey patched by eventlet/greenlet.
+    stdlib_threading = eventlet.patcher.original('threading')
+    stdlib_queue = eventlet.patcher.original('queue')
+else:
+    # Manage the case where we run this driver in a non patched environment
+    # and where user even so configure the driver to run heartbeat through
+    # a python thread, if we don't do that when the heartbeat will start
+    # we will facing an issue by trying to override the threading module.
+    stdlib_threading = threading
+    stdlib_queue = queue
+
 
 def version_is_compatible(imp_version, version):
     """Determine whether versions are compatible.
@@ -47,7 +69,7 @@ def version_is_compatible(imp_version, version):
     return True
 
 
-class DummyLock(object):
+class DummyLock:
     def acquire(self):
         pass
 
@@ -59,3 +81,11 @@ class DummyLock(object):
 
     def __exit__(self, type, value, traceback):
         self.release()
+
+
+def get_executor_with_context():
+    if eventletutils.is_monkey_patched('thread'):
+        LOG.debug("Threading is patched, using an eventlet executor")
+        return 'eventlet'
+    LOG.debug("Using a threading executor")
+    return 'threading'
