@@ -1,4 +1,3 @@
-
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -12,20 +11,14 @@
 #    under the License.
 
 import abc
-import logging
-
-import six
 
 
 __all__ = [
     "DispatcherBase"
 ]
 
-LOG = logging.getLogger(__name__)
 
-
-@six.add_metaclass(abc.ABCMeta)
-class DispatcherBase(object):
+class DispatcherBase(metaclass=abc.ABCMeta):
     "Base class for dispatcher"
 
     @abc.abstractmethod
