@@ -15,12 +15,10 @@
 
 import re
 
-import six
 
+class NotificationFilter:
 
-class NotificationFilter(object):
-
-    """Filter notification messages
+    r"""Filter notification messages
 
     The NotificationFilter class is used to filter notifications that an
     endpoint will received.
@@ -57,13 +55,13 @@ class NotificationFilter(object):
     def _build_regex_dict(regex_list):
         if regex_list is None:
             return {}
-        return dict((k, re.compile(regex_list[k])) for k in regex_list)
+        return {k: re.compile(regex_list[k]) for k in regex_list}
 
     @staticmethod
     def _check_for_single_mismatch(data, regex):
         if regex is None:
             return False
-        if not isinstance(data, six.string_types):
+        if not isinstance(data, str):
             return True
         if not regex.match(data):
             return True
