@@ -31,6 +31,7 @@ import oslo_messaging
 from oslo_messaging._drivers import amqpdriver
 from oslo_messaging._drivers import common as driver_common
 from oslo_messaging._drivers import impl_rabbit as rabbit_driver
+from oslo_messaging.exceptions import ConfigurationError
 from oslo_messaging.exceptions import MessageDeliveryFailure
 from oslo_messaging.tests import utils as test_utils
 from oslo_messaging.transport import DriverLoadFailure
@@ -205,6 +206,65 @@ class TestRabbitDriverLoadSSL(test_utils.BaseTestCase):
         )
 
 
+class TestRabbitDriverLoadSSLWithFIPS(test_utils.BaseTestCase):
+    scenarios = [
+        ('ssl_fips_mode', dict(options=dict(ssl=True,
+                                            ssl_enforce_fips_mode=True),
+                               expected=True)),
+    ]
+
+    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection'
+                '.ensure_connection')
+    @mock.patch('kombu.connection.Connection')
+    def test_driver_load_with_fips_supported(self,
+                                             connection_klass, fake_ensure):
+        self.config(ssl=True, ssl_enforce_fips_mode=True,
+                    group="oslo_messaging_rabbit")
+        transport = oslo_messaging.get_transport(self.conf,
+                                                 'kombu+memory:////')
+        self.addCleanup(transport.cleanup)
+
+        with mock.patch.object(ssl, 'FIPS_mode',
+                               create=True, return_value=True):
+            with mock.patch.object(ssl, 'FIPS_mode_set', create=True):
+
+                connection = transport._driver._get_connection()
+                connection_klass.assert_called_once_with(
+                    'memory:///', transport_options={
+                        'client_properties': {
+                            'capabilities': {
+                                'connection.blocked': True,
+                                'consumer_cancel_notify': True,
+                                'authentication_failure_close': True,
+                            },
+                            'connection_name': connection.name},
+                        'confirm_publish': True,
+                        'on_blocked': mock.ANY,
+                        'on_unblocked': mock.ANY},
+                    ssl=self.expected, login_method='AMQPLAIN',
+                    heartbeat=60, failover_strategy='round-robin'
+                )
+
+    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection'
+                '.ensure_connection')
+    @mock.patch('oslo_messaging._drivers.impl_rabbit.ssl')
+    @mock.patch('kombu.connection.Connection')
+    def test_fips_unsupported(self, connection_klass, fake_ssl, fake_ensure):
+        self.config(ssl=True, ssl_enforce_fips_mode=True,
+                    group="oslo_messaging_rabbit")
+        transport = oslo_messaging.get_transport(self.conf,
+                                                 'kombu+memory:////')
+        self.addCleanup(transport.cleanup)
+
+        del fake_ssl.FIPS_mode
+
+        # We do this test only if FIPS mode is not supported to
+        # ensure that we hard fail.
+        self.assertRaises(
+            ConfigurationError,
+            transport._driver._get_connection)
+
+
 class TestRabbitPublisher(test_utils.BaseTestCase):
     @mock.patch('kombu.messaging.Producer.publish')
     def test_send_with_timeout(self, fake_publish):
@@ -430,7 +490,7 @@ class TestRabbitTransportURL(test_utils.BaseTestCase):
     ]
 
     def setUp(self):
-        super(TestRabbitTransportURL, self).setUp()
+        super().setUp()
         self.messaging_conf.transport_url = 'rabbit:/'
         self.config(heartbeat_timeout_threshold=0,
                     group='oslo_messaging_rabbit')
@@ -608,7 +668,7 @@ class TestRacyWaitForReply(test_utils.BaseTestCase):
         wait_conditions = []
         orig_reply_waiter = amqpdriver.ReplyWaiter.wait
 
-        def reply_waiter(self, msg_id, timeout, call_monitor_timeout):
+        def reply_waiter(self, msg_id, timeout, call_monitor_timeout, reply_q):
             if wait_conditions:
                 cond = wait_conditions.pop()
                 with cond:
@@ -616,7 +676,7 @@ class TestRacyWaitForReply(test_utils.BaseTestCase):
                 with cond:
                     cond.wait()
             return orig_reply_waiter(self, msg_id, timeout,
-                                     call_monitor_timeout)
+                                     call_monitor_timeout, reply_q)
 
         self.useFixture(fixtures.MockPatchObject(
             amqpdriver.ReplyWaiter, 'wait', reply_waiter))
@@ -718,7 +778,7 @@ def _declare_queue(target):
                                          type='topic',
                                          durable=False,
                                          auto_delete=False)
-        topic = '%s.%s' % (target.topic, target.server)
+        topic = '{}.{}'.format(target.topic, target.server)
         queue = kombu.entity.Queue(name=topic,
                                    channel=channel,
                                    exchange=exchange,
@@ -780,7 +840,7 @@ class TestRequestWireFormat(test_utils.BaseTestCase):
                                                          cls._compression)
 
     def setUp(self):
-        super(TestRequestWireFormat, self).setUp()
+        super().setUp()
         self.uuids = []
         self.orig_uuid4 = uuid.uuid4
         self.useFixture(fixtures.MonkeyPatch('uuid.uuid4', self.mock_uuid4))
@@ -868,7 +928,7 @@ def _create_producer(target):
                                          type='topic',
                                          durable=False,
                                          auto_delete=False)
-        topic = '%s.%s' % (target.topic, target.server)
+        topic = '{}.{}'.format(target.topic, target.server)
         producer = kombu.messaging.Producer(exchange=exchange,
                                             channel=channel,
                                             routing_key=topic)
@@ -975,11 +1035,11 @@ TestReplyWireFormat.generate_scenarios()
 class RpcKombuHATestCase(test_utils.BaseTestCase):
 
     def setUp(self):
-        super(RpcKombuHATestCase, self).setUp()
+        super().setUp()
         transport_url = 'rabbit:/host1,host2,host3,host4,host5/'
         self.messaging_conf.transport_url = transport_url
-        self.config(rabbit_retry_interval=0.01,
-                    rabbit_retry_backoff=0.01,
+        self.config(rabbit_retry_interval=1.0,
+                    rabbit_retry_backoff=1.0,
                     kombu_reconnect_delay=0,
                     heartbeat_timeout_threshold=0,
                     group="oslo_messaging_rabbit")
