@@ -12,17 +12,18 @@
 
 import logging
 
+import fixtures
+
 import oslo_messaging
 from oslo_messaging.notify import log_handler
-from oslo_messaging.tests.notify import test_notifier
 from oslo_messaging.tests import utils as test_utils
-from six.moves import mock
+from unittest import mock
 
 
 class PublishErrorsHandlerTestCase(test_utils.BaseTestCase):
     """Tests for log.PublishErrorsHandler"""
     def setUp(self):
-        super(PublishErrorsHandlerTestCase, self).setUp()
+        super().setUp()
         self.publisherrorshandler = (log_handler.
                                      PublishErrorsHandler(logging.ERROR))
 
@@ -32,13 +33,14 @@ class PublishErrorsHandlerTestCase(test_utils.BaseTestCase):
                     group='oslo_messaging_notifications')
         self.stub_flg = True
 
-        transport = test_notifier._FakeTransport(self.conf)
+        transport = oslo_messaging.get_notification_transport(self.conf)
         notifier = oslo_messaging.Notifier(transport)
 
         def fake_notifier(*args, **kwargs):
             self.stub_flg = False
 
-        self.stubs.Set(notifier, 'error', fake_notifier)
+        self.useFixture(fixtures.MockPatchObject(
+            notifier, 'error', fake_notifier))
 
         logrecord = logging.LogRecord(name='name', level='WARN',
                                       pathname='/tmp', lineno=1, msg='Message',
