@@ -1,4 +1,3 @@
-
 # Copyright 2013 Red Hat, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,10 +13,11 @@
 #    under the License.
 
 from oslo_config import cfg
-from six.moves import mock
 import testscenarios
+from unittest import mock
 
 import oslo_messaging
+from oslo_messaging._metrics import client as metrics_client
 from oslo_messaging import exceptions
 from oslo_messaging import serializer as msg_serializer
 from oslo_messaging.tests import utils as test_utils
@@ -44,8 +44,9 @@ class TestCastCall(test_utils.BaseTestCase):
         self.config(rpc_response_timeout=None)
         transport_options = oslo_messaging.TransportOptions()
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
-        client = oslo_messaging.RPCClient(transport, oslo_messaging.Target(),
-                                          transport_options=transport_options)
+        client = oslo_messaging.get_rpc_client(
+            transport, oslo_messaging.Target(),
+            transport_options=transport_options)
 
         transport._send = mock.Mock()
 
@@ -70,7 +71,7 @@ class TestCastCall(test_utils.BaseTestCase):
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         transport_options = oslo_messaging.TransportOptions(at_least_once=True)
-        client = oslo_messaging.RPCClient(
+        client = oslo_messaging.get_rpc_client(
             transport,
             oslo_messaging.Target(),
             transport_options=transport_options)
@@ -208,14 +209,14 @@ class TestCastToTarget(test_utils.BaseTestCase):
                                                          cls._prepare)
 
     def setUp(self):
-        super(TestCastToTarget, self).setUp(conf=cfg.ConfigOpts())
+        super().setUp(conf=cfg.ConfigOpts())
 
     def test_cast_to_target(self):
         target = oslo_messaging.Target(**self.ctor)
         expect_target = oslo_messaging.Target(**self.expect)
 
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
-        client = oslo_messaging.RPCClient(transport, target)
+        client = oslo_messaging.get_rpc_client(transport, target)
 
         transport._send = mock.Mock()
 
@@ -269,9 +270,9 @@ class TestCallTimeout(test_utils.BaseTestCase):
         self.config(rpc_response_timeout=self.confval)
 
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
-        client = oslo_messaging.RPCClient(transport, oslo_messaging.Target(),
-                                          timeout=self.ctor,
-                                          call_monitor_timeout=self.cm)
+        client = oslo_messaging.get_rpc_client(
+            transport, oslo_messaging.Target(), timeout=self.ctor,
+            call_monitor_timeout=self.cm)
 
         transport._send = mock.Mock()
 
@@ -302,8 +303,9 @@ class TestCallRetry(test_utils.BaseTestCase):
 
     def test_call_retry(self):
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
-        client = oslo_messaging.RPCClient(transport, oslo_messaging.Target(),
-                                          retry=self.ctor)
+        client = oslo_messaging.get_rpc_client(
+            transport, oslo_messaging.Target(),
+            retry=self.ctor)
 
         transport._send = mock.Mock()
 
@@ -332,8 +334,8 @@ class TestCallFanout(test_utils.BaseTestCase):
 
     def test_call_fanout(self):
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
-        client = oslo_messaging.RPCClient(transport,
-                                          oslo_messaging.Target(**self.target))
+        client = oslo_messaging.get_rpc_client(
+            transport, oslo_messaging.Target(**self.target))
 
         if self.prepare is not _notset:
             client = client.prepare(**self.prepare)
@@ -363,8 +365,8 @@ class TestSerializer(test_utils.BaseTestCase):
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         serializer = msg_serializer.NoOpSerializer()
 
-        client = oslo_messaging.RPCClient(transport, oslo_messaging.Target(),
-                                          serializer=serializer)
+        client = oslo_messaging.get_rpc_client(
+            transport, oslo_messaging.Target(), serializer=serializer)
 
         transport._send = mock.Mock()
         kwargs = dict(wait_for_reply=True,
@@ -465,8 +467,8 @@ class TestVersionCap(test_utils.BaseTestCase):
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         target = oslo_messaging.Target(version=self.version)
-        client = oslo_messaging.RPCClient(transport, target,
-                                          version_cap=self.cap)
+        client = oslo_messaging.get_rpc_client(transport, target,
+                                               version_cap=self.cap)
 
         if self.success:
             transport._send = mock.Mock()
@@ -504,6 +506,7 @@ class TestVersionCap(test_utils.BaseTestCase):
                                                     transport_options=None,
                                                     **kwargs)
 
+
 TestVersionCap.generate_scenarios()
 
 
@@ -573,8 +576,8 @@ class TestCanSendVersion(test_utils.BaseTestCase):
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         target = oslo_messaging.Target(version=self.version)
-        client = oslo_messaging.RPCClient(transport, target,
-                                          version_cap=self.cap)
+        client = oslo_messaging.get_rpc_client(transport, target,
+                                               version_cap=self.cap)
 
         prep_kwargs = {}
         if self.prepare_cap is not _notset:
@@ -597,7 +600,7 @@ class TestCanSendVersion(test_utils.BaseTestCase):
     def test_invalid_version_type(self):
         target = oslo_messaging.Target(topic='sometopic')
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
-        client = oslo_messaging.RPCClient(transport, target)
+        client = oslo_messaging.get_rpc_client(transport, target)
         self.assertRaises(exceptions.MessagingException,
                           client.prepare, version='5')
         self.assertRaises(exceptions.MessagingException,
@@ -611,8 +614,70 @@ class TestTransportWarning(test_utils.BaseTestCase):
     @mock.patch('oslo_messaging.rpc.client.LOG')
     def test_warning_when_notifier_transport(self, log):
         transport = oslo_messaging.get_notification_transport(self.conf)
-        oslo_messaging.RPCClient(transport, oslo_messaging.Target())
+        oslo_messaging.get_rpc_client(transport, oslo_messaging.Target())
         log.warning.assert_called_once_with(
             "Using notification transport for RPC. Please use "
             "get_rpc_transport to obtain an RPC transport "
             "instance.")
+
+
+class TestClientMetrics(test_utils.BaseTestCase):
+    @mock.patch.object(metrics_client.MetricsCollectorClient,
+                       "put_into_txqueue")
+    def test_metrics(self, mock_put):
+        metrics = []
+
+        def fake_put_into_tx_queue(*args, **kwargs):
+            metrics.append((args, kwargs))
+
+        mock_put.side_effect = fake_put_into_tx_queue
+
+        metrics_client.METRICS_COLLECTOR = None
+        self.config(metrics_enabled=True, group="oslo_messaging_metrics")
+        self.ctxt = {}
+
+        transport_options = oslo_messaging.TransportOptions()
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
+        transport._send = mock.Mock()
+        target = oslo_messaging.Target(topic="topic1")
+
+        self.client = oslo_messaging.RPCClient(
+            transport, target,
+            transport_options=transport_options)
+
+        self.client.call(self.ctxt, 'foo')
+
+        self.assertTrue(metrics)
+        for _args, labels in metrics:
+            self.assertEqual(
+                {
+                    'call_type': 'call',
+                    'exchange': None,
+                    'fanout': None,
+                    'method': 'foo',
+                    'namespace': None,
+                    'server': None,
+                    'timeout': None,
+                    'topic': 'topic1',
+                    'version': None,
+                }, labels)
+
+        metrics = []
+
+        self.client.target.topic = 'topic2'
+        self.client.cast(self.ctxt, 'bar')
+
+        self.assertTrue(metrics)
+        for _args, labels in metrics:
+            self.assertEqual(
+                {
+                    'call_type': 'cast',
+                    'exchange': None,
+                    'fanout': None,
+                    'method': 'bar',
+                    'namespace': None,
+                    'server': None,
+                    'timeout': None,
+                    'topic': 'topic2',
+                    'version': None,
+                }, labels)
