@@ -1,4 +1,3 @@
-
 # Copyright 2013 Red Hat, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,9 +13,9 @@
 #    under the License.
 
 import fixtures
+from unittest import mock
+
 from oslo_config import cfg
-import six
-from six.moves import mock
 from stevedore import driver
 import testscenarios
 
@@ -27,7 +26,7 @@ from oslo_messaging import transport
 load_tests = testscenarios.load_tests_apply_scenarios
 
 
-class _FakeDriver(object):
+class _FakeDriver:
 
     def __init__(self, conf):
         self.conf = conf
@@ -42,7 +41,7 @@ class _FakeDriver(object):
         pass
 
 
-class _FakeManager(object):
+class _FakeManager:
 
     def __init__(self, driver):
         self.driver = driver
@@ -51,36 +50,36 @@ class _FakeManager(object):
 class GetTransportTestCase(test_utils.BaseTestCase):
 
     scenarios = [
-        ('rpc_backend',
-         dict(url=None, transport_url=None, rpc_backend='testbackend',
+        ('default',
+         dict(url=None, transport_url=None,
               control_exchange=None, allowed=None,
-              expect=dict(backend='testbackend',
+              expect=dict(backend='rabbit',
                           exchange=None,
-                          url='testbackend:',
+                          url='rabbit:',
                           allowed=[]))),
         ('transport_url',
-         dict(url=None, transport_url='testtransport:', rpc_backend=None,
+         dict(url=None, transport_url='testtransport:',
               control_exchange=None, allowed=None,
               expect=dict(backend='testtransport',
                           exchange=None,
                           url='testtransport:',
                           allowed=[]))),
         ('url_param',
-         dict(url='testtransport:', transport_url=None, rpc_backend=None,
+         dict(url='testtransport:', transport_url=None,
               control_exchange=None, allowed=None,
               expect=dict(backend='testtransport',
                           exchange=None,
                           url='testtransport:',
                           allowed=[]))),
         ('control_exchange',
-         dict(url=None, transport_url=None, rpc_backend='testbackend',
+         dict(url=None, transport_url='testbackend:',
               control_exchange='testexchange', allowed=None,
               expect=dict(backend='testbackend',
                           exchange='testexchange',
                           url='testbackend:',
                           allowed=[]))),
         ('allowed_remote_exmods',
-         dict(url=None, transport_url=None, rpc_backend='testbackend',
+         dict(url=None, transport_url='testbackend:',
               control_exchange=None, allowed=['foo', 'bar'],
               expect=dict(backend='testbackend',
                           exchange=None,
@@ -90,9 +89,10 @@ class GetTransportTestCase(test_utils.BaseTestCase):
 
     @mock.patch('oslo_messaging.transport.LOG')
     def test_get_transport(self, fake_logger):
-        self.config(rpc_backend=self.rpc_backend,
-                    control_exchange=self.control_exchange,
-                    transport_url=self.transport_url)
+        self.messaging_conf.reset()
+        self.config(control_exchange=self.control_exchange)
+        if self.transport_url:
+            self.config(transport_url=self.transport_url)
 
         driver.DriverManager = mock.Mock()
 
@@ -114,7 +114,7 @@ class GetTransportTestCase(test_utils.BaseTestCase):
         self.assertIsNotNone(transport_)
         self.assertIs(transport_.conf, self.conf)
         self.assertIs(transport_._driver, drvr)
-        self.assertTrue(isinstance(transport_, transport.RPCTransport))
+        self.assertIsInstance(transport_, transport.RPCTransport)
 
         driver.DriverManager.assert_called_once_with('oslo.messaging.drivers',
                                                      self.expect['backend'],
@@ -127,76 +127,51 @@ class GetTransportSadPathTestCase(test_utils.BaseTestCase):
 
     scenarios = [
         ('invalid_transport_url',
-         dict(url=None, transport_url='invalid', rpc_backend=None,
+         dict(url=None, transport_url='invalid',
               ex=dict(cls=oslo_messaging.InvalidTransportURL,
                       msg_contains='No scheme specified',
                       url='invalid'))),
         ('invalid_url_param',
-         dict(url='invalid', transport_url=None, rpc_backend=None,
+         dict(url='invalid', transport_url=None,
               ex=dict(cls=oslo_messaging.InvalidTransportURL,
                       msg_contains='No scheme specified',
                       url='invalid'))),
         ('driver_load_failure',
-         dict(url=None, transport_url=None, rpc_backend='testbackend',
+         dict(url=None, transport_url='testbackend:/',
               ex=dict(cls=oslo_messaging.DriverLoadFailure,
                       msg_contains='Failed to load',
                       driver='testbackend'))),
     ]
 
     def test_get_transport_sad(self):
-        self.config(rpc_backend=self.rpc_backend,
-                    transport_url=self.transport_url)
-
-        if self.rpc_backend:
-            driver.DriverManager = mock.Mock()
-
-            invoke_args = [self.conf,
-                           oslo_messaging.TransportURL.parse(self.conf,
-                                                             self.url)]
-            invoke_kwds = dict(default_exchange='openstack',
-                               allowed_remote_exmods=[])
-
-            driver.DriverManager.side_effect = RuntimeError()
-        try:
-            oslo_messaging.get_transport(self.conf, url=self.url)
-            self.assertFalse(True)
-
-            driver.DriverManager.\
-                assert_called_once_with('oslo.messaging.drivers',
-                                        self.rpc_backend,
-                                        invoke_on_load=True,
-                                        invoke_args=invoke_args,
-                                        invoke_kwds=invoke_kwds)
-        except Exception as ex:
-            ex_cls = self.ex.pop('cls')
-            ex_msg_contains = self.ex.pop('msg_contains')
-
-            self.assertIsInstance(ex, oslo_messaging.MessagingException)
-            self.assertIsInstance(ex, ex_cls)
-            self.assertIn(ex_msg_contains, six.text_type(ex))
-
-            for k, v in self.ex.items():
-                self.assertTrue(hasattr(ex, k))
-                self.assertEqual(v, str(getattr(ex, k)))
+        self.config(transport_url=self.transport_url)
+        ex_cls = self.ex.pop('cls')
+        ex_msg_contains = self.ex.pop('msg_contains')
+        ex = self.assertRaises(
+            ex_cls, oslo_messaging.get_transport, self.conf, url=self.url)
+        self.assertIn(ex_msg_contains, str(ex))
+        for k, v in self.ex.items():
+            self.assertTrue(hasattr(ex, k))
+            self.assertEqual(v, str(getattr(ex, k)))
 
 
 # FIXME(markmc): this could be used elsewhere
 class _SetDefaultsFixture(fixtures.Fixture):
 
     def __init__(self, set_defaults, opts, *names):
-        super(_SetDefaultsFixture, self).__init__()
+        super().__init__()
         self.set_defaults = set_defaults
         self.opts = opts
         self.names = names
 
     def setUp(self):
-        super(_SetDefaultsFixture, self).setUp()
+        super().setUp()
 
         # FIXME(markmc): this comes from Id5c1f3ba
         def first(seq, default=None, key=None):
             if key is None:
                 key = bool
-            return next(six.moves.filter(key, seq), default)
+            return next(filter(key, seq), default)
 
         def default(opts, name):
             return first(opts, key=lambda o: o.name == name).default
@@ -214,7 +189,7 @@ class _SetDefaultsFixture(fixtures.Fixture):
 class TestSetDefaults(test_utils.BaseTestCase):
 
     def setUp(self):
-        super(TestSetDefaults, self).setUp(conf=cfg.ConfigOpts())
+        super().setUp(conf=cfg.ConfigOpts())
         self.useFixture(_SetDefaultsFixture(
             oslo_messaging.set_transport_defaults,
             transport._transport_opts,
@@ -255,7 +230,8 @@ class TestTransportMethodArgs(test_utils.BaseTestCase):
                                                wait_for_reply=None,
                                                timeout=None,
                                                call_monitor_timeout=None,
-                                               retry=None)
+                                               retry=None,
+                                               transport_options=None)
 
     def test_send_all_args(self):
         t = transport.Transport(_FakeDriver(cfg.CONF))
@@ -274,7 +250,8 @@ class TestTransportMethodArgs(test_utils.BaseTestCase):
                                     wait_for_reply='wait_for_reply',
                                     timeout='timeout',
                                     call_monitor_timeout='cm_timeout',
-                                    retry='retry')
+                                    retry='retry',
+                                    transport_options=None)
 
     def test_send_notification(self):
         t = transport.Transport(_FakeDriver(cfg.CONF))
@@ -315,7 +292,7 @@ class TestTransportMethodArgs(test_utils.BaseTestCase):
 
 class TestTransportUrlCustomisation(test_utils.BaseTestCase):
     def setUp(self):
-        super(TestTransportUrlCustomisation, self).setUp()
+        super().setUp()
 
         def transport_url_parse(url):
             return transport.TransportURL.parse(self.conf, url)
@@ -327,7 +304,7 @@ class TestTransportUrlCustomisation(test_utils.BaseTestCase):
             "fake:/vhost1/localhost:5672/?l=1&l=2&l=3")
         self.url4 = transport_url_parse(
             "fake:/vhost2/localhost:5672/?d=x:1&d=y:2&d=z:3")
-        self.url5 = transport_url_parse("fake://noport:/?")
+        self.url5 = transport_url_parse("fake://noport/?")
 
     def test_hash(self):
         urls = {}
@@ -355,7 +332,7 @@ class TestTransportUrlCustomisation(test_utils.BaseTestCase):
 
 class TestTransportHostCustomisation(test_utils.BaseTestCase):
     def setUp(self):
-        super(TestTransportHostCustomisation, self).setUp()
+        super().setUp()
         self.host1 = transport.TransportHost("host1", 5662, "user", "pass")
         self.host2 = transport.TransportHost("host1", 5662, "user", "pass")
         self.host3 = transport.TransportHost("host1", 5663, "user", "pass")
