@@ -1,4 +1,3 @@
-
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -21,6 +20,7 @@ import logging
 
 from debtcollector import removals
 from oslo_config import cfg
+from oslo_utils import netutils
 from stevedore import driver
 from urllib import parse
 
@@ -72,7 +72,7 @@ def set_transport_defaults(control_exchange):
                      control_exchange=control_exchange)
 
 
-class Transport(object):
+class Transport:
 
     """A messaging transport.
 
@@ -162,21 +162,21 @@ class RPCTransport(Transport):
     """Transport object for RPC."""
 
     def __init__(self, driver):
-        super(RPCTransport, self).__init__(driver)
+        super().__init__(driver)
 
 
 class NotificationTransport(Transport):
     """Transport object for notifications."""
 
     def __init__(self, driver):
-        super(NotificationTransport, self).__init__(driver)
+        super().__init__(driver)
 
 
 class InvalidTransportURL(exceptions.MessagingException):
     """Raised if transport URL is invalid."""
 
     def __init__(self, url, msg):
-        super(InvalidTransportURL, self).__init__(msg)
+        super().__init__(msg)
         self.url = url
 
 
@@ -184,8 +184,8 @@ class DriverLoadFailure(exceptions.MessagingException):
     """Raised if a transport driver can't be loaded."""
 
     def __init__(self, driver, ex):
-        msg = 'Failed to load transport driver "%s": %s' % (driver, ex)
-        super(DriverLoadFailure, self).__init__(msg)
+        msg = 'Failed to load transport driver "{}": {}'.format(driver, ex)
+        super().__init__(msg)
         self.driver = driver
         self.ex = ex
 
@@ -248,7 +248,7 @@ def get_transport(conf, url=None, allowed_remote_exmods=None):
                           transport_cls=RPCTransport)
 
 
-class TransportHost(object):
+class TransportHost:
 
     """A host element of a parsed transport URL."""
 
@@ -277,7 +277,7 @@ class TransportHost(object):
         return '<TransportHost ' + values + '>'
 
 
-class TransportOptions(object):
+class TransportOptions:
 
     def __init__(self, at_least_once=False):
         self._at_least_once = at_least_once
@@ -287,7 +287,7 @@ class TransportOptions(object):
         return self._at_least_once
 
 
-class TransportURL(object):
+class TransportURL:
 
     """A parsed transport URL.
 
@@ -397,17 +397,14 @@ class TransportURL(object):
 
             # Build the network location portion of the transport URL
             if hostname:
-                if ':' in hostname:
-                    netloc += '[%s]' % hostname
-                else:
-                    netloc += hostname
+                netloc += netutils.escape_ipv6(hostname)
             if port is not None:
                 netloc += ':%d' % port
 
             netlocs.append(netloc)
 
         # Assemble the transport URL
-        url = '%s://%s/' % (self.transport, ','.join(netlocs))
+        url = '{}://{}/'.format(self.transport, ','.join(netlocs))
 
         if self.virtual_host:
             url += parse.quote(self.virtual_host)
@@ -492,31 +489,7 @@ class TransportURL(object):
                     password = parse.unquote(password)
                 username = parse.unquote(username)
 
-            if not hostname:
-                hostname = None
-            elif hostname.startswith('['):
-                # Find the closing ']' and extract the hostname
-                host_end = hostname.find(']')
-                if host_end < 0:
-                    # NOTE(Vek): Identical to what Python 2.7's
-                    # urlparse.urlparse() raises in this case
-                    raise ValueError('Invalid IPv6 URL')
-
-                port_text = hostname[host_end:]
-                hostname = hostname[1:host_end]
-
-                # Now we need the port; this is compliant with how urlparse
-                # parses the port data
-                port = None
-                if ':' in port_text:
-                    port = port_text.split(':', 1)[1]
-            elif ':' in hostname:
-                hostname, port = hostname.split(':', 1)
-
-            if port == "":
-                port = None
-            if port is not None:
-                port = int(port)
+            hostname, port = netutils.parse_host_port(hostname)
 
             if username is None or password is None:
                 hosts_without_credentials.append(hostname)
